#include "Game.h"

// Constructor
Game::Game(std::string name, int player_amount) {
	m_name = name;
	m_player_amount = player_amount;
	m_dealer = nullptr;
	m_deck = nullptr;
}

// Constructor
Game::Game(std::string name, int player_amount, Dealer *dealer) {
	m_name = name;
	m_player_amount = player_amount;
	m_dealer = dealer;
	m_deck = nullptr;
}

// Destructor
Game::~Game() {
	for (Player* p : m_players) {
		delete p;
	}
	delete m_deck;

	std::cout << "game destroyed!" << std::endl;
}

// Add a pointer to a dealer object
void Game::AddDealer(Dealer* dealer) {
	try {
		if (dealer->GetLicense()) {
			m_dealer = dealer;
		}
		else throw(dealer->GetName());
	}
	catch (std::string name) {
		std::cout << "CANNOT ADD THE DEALER TO A GAME" << std::endl;
		std::cout << "Dealer : " << name << " has no valid license." << std::endl;
	}
}

/// <summary>
///  Add a player to the table or game
/// Check if the player is old enough and that the table has room
/// </summary>
/// <param name="player"></param>
bool Game::AddPlayer(Player* player) {
	try {
		if (player->GetAge() >= AGE_LIMIT) {
			try {
				if (m_players.size() >= m_player_amount) throw(player->GetName());
				m_players.push_back(player);
				return true;

			} catch(std::string name) {
				std::cout << "CANNOT ADD THE PLAYER TO A GAME" << std::endl;
				std::cout << "The table is currently full." << std::endl;
				return false;
			}
		}
		else throw(player->GetName());
	}
	catch (std::string name) {
		std::cout << "CANNOT ADD THE PLAYER TO A GAME" << std::endl;
		std::cout << "Player " << name << " is not old enough." << std::endl;
		return false;
	}
}

// Add a deck to the game
void Game::AddDeck(Deck* deck) {
	m_deck = deck;
}

/// <summary>
/// Prints the game information, the game name
/// the dealer, max player amount and players currently in the table
/// </summary>
void Game::PrintGameInfo() const {
	std::cout << "Game: " << m_name << std::endl;
	std::cout << "Max Players: " << m_player_amount << std::endl;

	if(m_dealer != nullptr) std::cout << "Dealer: " << m_dealer->GetName() << std::endl;

	std::cout << "CURRENT PLAYERS IN THE TABLE ARE : " << std::endl;
	for (Player* p : m_players) {
		std::cout << p->GetName() << std::endl;
	}
}

/// <summary>
/// Shuffles the games deck
/// </summary>
void Game::ShuffleTheDeck() {
	std::cout << "The dealer " << m_dealer->GetName() << " shuffles the deck." << std::endl;
	m_deck->Shuffle();
}

/// <summary>
/// Asks players to place a bet
/// The bet is stored on the player objects
/// </summary>
void Game::PlaceBets() {
	std::cout << "TIME TO PLACE YOUR BETS!!" << std::endl;
	for (Player* p : m_players) {
		std::cout << "What is your bet " << p->GetName() << "?" << std::endl;
		int bet;
		std::cin >> bet;
		p->SetBet(bet);
	}
}

/// <summary>
/// Shows the hands of the players and then the dealers
/// </summary>
void Game::ShowHands() const {
	std::cout << "                             LETS SHOW HANDS !!!!                          " << std::endl;
	std::cout << "---------------------------------------------------------------------------" << std::endl;

	for (Player* p : m_players) {
		p->ShowHand();
	}

	m_dealer->ShowCard();
}

void Game::DealerWon() {
	std::cout << "DEALER WON WITH :" << std::endl;
	m_dealer->ShowHand();
}

void Game::PlayerWon(Player* player) {
	std::cout << "PLAYER " << player->GetName() << " WON WITH :" << std::endl;
	player->ShowHand();
}

/// <summary>
/// Starts the round with placing bets and dealing the cards to the players
/// Then checks if anyone got a blackjack
/// </summary>
void Game::StartRound() {
	PlaceBets();
	m_dealer->DealCards(*m_deck, m_players);
	ShowHands();
	
	if (m_dealer->IsBlackjack()) {
		DealerWon();
		return;
	}
	else {
		for (Player* p : m_players) {
			if (p->IsBlackjack()) {
				PlayerWon(p);
			}
		}
	}
	m_dealer->ShowHand();
}

/// <summary>
/// This is where the game progression is and the game logic
/// Game is started and players are recruited by the terminal
/// TODO: Error handling for cin invalid inputs
/// </summary>
void Game::Start() {
	std::cout << "|--------------------------------------------------------------------------------------|" << std::endl;
	std::cout << "|                                  WELCOME TO BLACKJACK!!                              |" << std::endl;
	std::cout << "|--------------------------------------------------------------------------------------|" << std::endl;

	Dealer dealer("Steve", 45);
	AddDealer(&dealer);
	AddDeck(new Deck());
	
	std::cout << "Your dealers name is : " << dealer.GetName() << std::endl;
	std::cout << std::endl;
	int player_count = 0;
	while (true) {
		std::cout << "How many players? Maximum of " << m_player_amount << std::endl;
		std::cin >> player_count;
		
		if (player_count <= m_player_amount && player_count > 0) break;
	}
	
	for (int i = 0; i < player_count; i++) {
		std::cout << "What is your name player " << i+1 << "?" << std::endl;
		std::string name;
		int age;
		std::cin >> name;
		std::cout << "How old are you?" << std::endl;
		std::cin >> age;
		Player* player = new Player(name, age);
		if(AddPlayer(player)) std::cout << "Welcome to the table to play Blackjack, " << player->GetName() << std::endl;
	}

	ShuffleTheDeck();
	StartRound();
	// TODO: Take the cards from the players


	std::cout << "|--------------------------------------------------------------------------------------|" << std::endl;
	std::cout << "|                                         GAME OVER!!                                  |" << std::endl;
	std::cout << "|--------------------------------------------------------------------------------------|" << std::endl;
}

